// "The two-loop helicity amplitudes for gg -> V1V2 -> 4 leptons"
// A. von Manteuffel, L. Tancredi
//
// ggvvamp package
//
// ggvvampprec: compute amplitude coefficients E with precision control (recommended)


#ifndef GGVVAMPPREC_H
#define GGVVAMPPREC_H

#include "ggvvamp.h"
#include <map>

const int nEhel = 2;  // gluon helicities {0=LL, 1=LR}
const int nEcoeff = 9; // coefficient type for fixed helicities {1,...,9}

// The code below assumes double and quad precision to be available.
// If required also other types could be used.
#if defined(DOUBLE_PRECISION) && defined(QUAD_PRECISION)

// amplitude tensor coefficients for g g -> V V' with numerical precision control
/* The idea is to compare results obtained with different precision settings
 * and increase the precision used in the evaluation until results are
 * consistent within a user-defined maximal uncertainty. */
class ggvvampprec {
public:
  // construct instance:
  /* Nf is number of flavors in closed loops
   * ir the IR subtraction scheme to be used for the result (see ggvvamp.h) */
  ggvvampprec(int Nf, IRscheme ir);
  // evaluate amplitude coefficients in a reliable way
  /* s = (p1+p2)^2, t = (p1-p3)^2, ma2 = p3^2, mb2 = p4^2 */
  void compute(double s, double t, double ma2, double mb2);
  // output some statistis
  void print_statistics(std::ostream&) const;
 
  double maxrel; // maximal relative target uncertainty
  double maxabs; // maximal absolute target uncertainty
   // best result for amplitude coefficients E (see paper for definition)
  double E[nEhel][nEcoeff+1][nloop][nreim]; // first index: 0..1, second index: 1..9
  std::map<long, long> eventsWithDigits; // counters for some statistics

private:
  ggvvamp<double>    d, dx12, dx34, dx12x34; // double precision evaluation
  ggvvamp<quadtype>  q, qx12, qx34, qx12x34; // quad precision evaluation
#ifdef ARBITRARY_PRECISION
  ggvvamp<cln::cl_F> a, ax12, ax34, ax12x34; // arbitrary precision evaluation
#endif
};
#endif

#endif // GGVVAMPPREC_H
